#! /usr/local/bin/pythone

__author__ = 'Marti Hearst'
__python_version = '3.2.2'

"""
xmlTreeSamples.py

Show how to use ElementTree objects for parsing and manipulating XML.
Created by Marti Hearst on 2012-02-25.
"""
import os

from xml.etree import ElementTree 


# assume xml data file in same directory as the python source file
# this is an easy way to get the right path name for the xml file
def getLocalPathFromThisSourceFile(fname):
	curpath = os.path.abspath(__file__)
	xml_file = os.path.dirname(curpath)
	xml_file = os.path.join(xml_file, fname)
	return xml_file

# print out the node; ignoring the tail 
def printXNodeContents(node, indent):
	if node is None: return
	print(indent + "Tag: " + str(node.tag)  + " Attrib: " + str(node.attrib))

# Print the contents of the tree 
# The indent code is a trick to make the tree indent farther to the right for each level
# python lets you multiply a string by a number to make the string repeat
def printXNode(node, level):
	if node is None: return
	indent = "   "*level
	printXNodeContents(node, indent)
	for child in node.getchildren():
		printXNode(child, level+1)

# insert a node as the first child under the matching key attribute
def insertXNode(node, key, value, newNode):
	if node is None: return
	if node.get(key) == value:
		node.insert(0, newNode)
		return
	for child in node.getchildren():
		insertXNode(child, key, value, newNode)
		
	
# Load and parse an XML file, print it, return it.
def parseXml(fname):
	try:
		xml_file = getLocalPathFromThisSourceFile(fname) 
		tree = ElementTree.parse(xml_file)
	except Exception as inst:
		print ("error opening file: %s", inst)
		return
	return tree

#  This function is for the optional exercise only.
#  This should get you started on building your own tree from a file.
#  Here I just created a list of elements.
#  You need to read in the values to use to create the elements
#  and assign them to the new tree.	You need to expand on this code
#  to use more than just tags, since you need attributes too.
#  Instead of giving a preset list, your code should read the
#  components of the hierarchy from an input file.

def buildXTree():
	
	# create root
	root = ElementTree.Element("root")
	# create some nodes; 
	newElements = [ElementTree.Element(tag) for tag in ["tag1", "tag2", "tag3"]]
	# Inserting all nodes at the same level; 
	# You need to change this code so that you have sublevels as well
	for newEl in newElements:
		root.insert(0, newEl)	
	# Make this call at the end to create the new tree
	newtree = ElementTree.ElementTree(root)
	printXNode(newtree.getroot(), 0)
	return newtree
	
# Examples of using the ElementTree package.

def tryOtherFunctions(root):
	# find the nodes with the tag taxonomy
	results = root.findall("taxonomy")
	print("Number of elements with tag taxonomy is: " + str(len(results)))
	# find the nodes in a longer path
	results = root.findall("taxonomy/heading")
	print("Number of elements with tag taxonomy/heading is: " + str(len(results)))
	
	# get all nodes
	allnodes = root.getiterator()
	print("Total number of elements is: " + str(len(allnodes)))
	
	# look for information about a node with a particular id
	for el in allnodes:
		if el.get('id') == "ca": printXNodeContents(el ," ")
		
	# Create a new node to insert.  Make it a new French region (Paris).
	dict = {"id": "par", "title": "Paris"}	
	newNode = ElementTree.Element("heading", dict)	
	# insert it under the title French, in the first position
	insertXNode(root, "title", "French", newNode)
	# make sure it worked
	printXNode(root, 0)


def main():
	tree = parseXml("wineabbrev.xml")
	if tree is not None:
		root = tree.getroot()
		print("Printing entire tree:")
		printXNode(root, 0)
		tryOtherFunctions(root)

if __name__ == '__main__':
	main()

	

		
